// msine2.c
// Monitor Test Program: Lissajous using Points (for slow CPUs)
// Copyright 1998  Jed Margolin

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <process.h>
#include <graphics.h>
#include <math.h>

#define XOFFSET 320		// screen x offset
#define YOFFSET 180		// screen y offset
#define SCALE 256		   // screen scaling for points
#define SPEED 4  			// adjust control sensitivity
#define DEG 512			// circle divider, binary multiples only

#define RETURN 13
#define UPARROW 72
#define DOWNARROW 80
#define LEFTARROW 75
#define RIGHTARROW 77
#define END 79
#define PgUp 73
#define PgDn 81
#define CTL_PgUp 132
#define CTL_PgDn 118
#define ESC 27
#define FF 13
#define CR 10
#define TRUE 1

extern void wait_for_retrace(void);
extern int scolor,bkcolor;

int psin_table[DEG];

void sine2(void);
void waveb1(int);
void waveb2(int);
void points_program(int,int),rpoints_program(int,int);
void spoints_program(int,int),rspoints_program(int,int);
void npoints_program(int,int);
// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
void sine2()
{
	int driver, mode, n, dquit;
	double val = 0.0;
	int page,mscale,bscale;
	unsigned char c;

	bkcolor = BLACK;
	scolor = GREEN;

	driver = VGA;
	mode = VGAMED;
	registerbgidriver(EGAVGA_driver);
	initgraph(&driver, &mode, "");

	setbkcolor(bkcolor);
	setcolor(scolor);
//-------------------------------------------------------------------------
// Make the sine table
	for(n=0;n<DEG;n++)
	{
		psin_table[n] =  (sin(val*3.14159/180.0) * 32000/SCALE);
		val += 360.0/DEG;
	}

	page = 0;
	mscale = 3;
	bscale = 1;
// ---------------------------------------------------------------------
// Read and process keyboard
	dquit = 0;

	while(!(dquit))
	{
		if(page == 0)	page = 1;	// video page ping-pong
		else		page = 0;

		setactivepage(page);  // directs the output of the graphics functions
		cleardevice();        // clears the screen (visual page)
		setcolor(scolor);

		if(bscale>0)
		{
			spoints_program(mscale,bscale);	// sine program
		}
		else
		{
			rspoints_program(mscale,bscale);	// rsine program
		}

		setvisualpage(page);		// displays the video page
		wait_for_retrace();

		if( kbhit() )
		{
			c = getch() ;
			if (!c) c = getch() ;	// get second byte if cursor key
			switch (c)
			{
			  case RIGHTARROW :
				  mscale++;
				  break ;

			  case LEFTARROW :
				  mscale--;
				  break ;

			  case UPARROW :
				  bscale++;
				  if(bscale==0)   bscale = 2;
				  break ;

			  case DOWNARROW :
				  bscale--;
				  if(bscale==0)   bscale = -2;
				  break ;

			  case PgDn :
				  scolor--;
				  scolor = scolor&0x0F;
				  break ;

			  case PgUp :
				  scolor++;
				  scolor = scolor&0x0F;
				  break ;

			  case CTL_PgDn :
				  bkcolor--;
				  bkcolor = bkcolor&0x0F;
				  setbkcolor(bkcolor);
				  break ;

			  case CTL_PgUp :
				  bkcolor++;
				  bkcolor = bkcolor&0x0F;
				  setbkcolor(bkcolor);
				  break ;

			  case RETURN :
				  dquit = TRUE;
				  break ;

			  case ESC :
				  dquit = TRUE;
				  break ;
			}	  // end of switch
		}	  // end if
	}	  	// end while

	restorecrtmode();
}
// end of program
// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
// put out slow points
void spoints_program(int mscale, int bscale)
{
	int i,sx,sy;
	static int m=0;

	for(i=0;i<DEG;i++)
	{
		sx=(3*psin_table[(i&(DEG-1))]>>1) + XOFFSET;
		sy=(psin_table[ (bscale*i+m/SPEED)&(DEG-1) ]) + YOFFSET;
		putpixel(sx,sy,scolor);
	}

	m = (m + mscale) & (SPEED*DEG-1);
}
// ---------------------------------------------------------------------
// ---------------------------------------------------------------------
// put out reverse slow points
void rspoints_program(int mscale, int bscale)
{
	int i,sx,sy;
	static int m=0;

	for(i=0;i<(DEG);i++)
	{
		sx=(3*psin_table[(bscale*i+m/SPEED)&(DEG-1)]>>1) + XOFFSET;
		sy=(psin_table[(i&(DEG-1))]) + YOFFSET;
		putpixel(sx,sy,scolor);
	}

	m = (m + mscale) & (SPEED*DEG-1);
}
// ---------------------------------------------------------------------
